import pandas as pd
import hashlib

from ..wizrenderer import WizRenderer
from ..dtype import *
from ..configs import *

from .calc import calc
from .wiz import wiz
 


def eda(df, col1=None, col2=None, vars=None, cache=None, dash=True,stats=False,corr=False):
    
    # Disable cache if dash == 'only'
    if dash == 'only': cache = False

    # Disable cache if dash == 'only'
    if corr: cache = not corr

    
    # Configuration settings
    cfg = Configs.current_theme

    # Get dtypes
    dtypes = Configs.get_dtypes(df)

    # Check if caching is enabled
    use_cache = Configs.use_eda_cache if cache is None else cache

    

    # get the size of eda_cache dictionary
    memory_size = sys.getsizeof(Configs.eda_cache)
    
    
    # Generate a hash key for caching based on function parameters
    cache_key = hashlib.sha256(f"{col1}-{col2}-{dtypes}".encode()).hexdigest()

    # Check if result is cached
    if use_cache and cache_key in Configs.eda_cache:
        cached_result = Configs.eda_cache[cache_key]
        cached_df = cached_result['df']

        # compare the two DataFrames
        if col1 is None:
            are_equal = df.equals(cached_df)
        elif col1 == 'WRAPPER':
            are_equal = df.equals(cached_df)
        else:
            are_equal = df[col1].equals(cached_df[col1])
        if are_equal:
            # Return the cached result
            return WizRenderer(cached_result['data_load'], cached_result['variable_type'], Configs.current_theme)

    # Calculate
    hub = calc(df, col1, col2, cfg, dtypes, vars, dash,stats,corr)
    #print(hub)
    # hub.save()
    if hub is None:  # because plot(df) is recursive
        return 0

    # Render
    data_load = wiz(hub, cfg)
    
    # Save the result to cache
    if use_cache:
        Configs.eda_cache[cache_key] = {
            'data_load': data_load,
            'variable_type': hub.variable_type,
            'df': df.copy()  # Cache a copy of the DataFrame
        }


    return WizRenderer(data_load, hub.variable_type, cfg)



import os
from os.path import dirname, join
import pandas as pd


def load_dataset(name: str or None):
    """
    Load a dataset by name or list all available datasets.
    
    name = 'all' -> lists all available datasets
    name = 'titanic' -> import titanic dataset
    
    Parameters:
        name (str or None): The name of the dataset to import.
                            Use 'all' to list all available datasets.
    
    Returns:
        pandas.DataFrame or List[str]: The imported dataset as a pandas DataFrame
                                       if name is provided.
                                       List of available datasets
                                       if name is None or 'all'.
    
    Raises:
        ValueError: If the specified dataset name is not found.
    """
    if name == 'all':
        return _list_available_datasets()
    else:
        path = _get_dataset_path_by_name(name)
        return pd.read_csv(path, engine='python',on_bad_lines='skip')


def _get_dataset_path_by_name(name: str):
    dir_path = dirname(__file__)
    files = [f for f in os.listdir(dir_path) if f.endswith(".csv")]

    lowercase = name.lower()
    if lowercase.endswith(".csv"):
        lowercase = os.path.splitext(lowercase)[0]

    if lowercase not in [os.path.splitext(f)[0] for f in files]:
        raise ValueError(f"Dataset {name} is not in the file.")

    return join(dir_path, f"{lowercase}.csv")


def _list_available_datasets():
    dir_path = dirname(__file__)
    files = [f for f in os.listdir(dir_path) if f.endswith(".csv")]
    return [os.path.splitext(f)[0] for f in files]
